#include "../include/troops_attacking.h"
#include "../include/fighter_list.h"

int handle_currentAttacks(MYSQL* currentDB)
{
    MYSQL_RES* resultset;
    MYSQL_ROW row;
    char* query;
    long len;
    
    /*create tmp and insert lines with wide attacks*/
    if(! (administrate_db(currentDB, Q700) ))
	return -1;
    /*insert lines attacking buildings_ready*/
    if(! (administrate_db(currentDB, Q701) ))
	return -1;
    /*insert lines attacking troops*/
    if(! (administrate_db(currentDB, Q702) ))
	return -1;
    /*insert lines attacking bundles*/
    if(! (administrate_db(currentDB, Q703) ))
	return -1;
    /*get all current attacks*/
    if( (resultset=(MYSQL_RES*) select_result(currentDB, Q704)) == NULL)
	return -1;
    /*
	0 attID
	1 myBundleID
	2 classID
	3 ownerID_target
	4 buildID_enemy
	5 troopID_enemy
	6 bundleID_enemy
	7 sequence_number for checks or whatever
	8 ownerID_current_pos to make life easiear
    */
	
    while(row=(MYSQL_ROW)mysql_fetch_row(resultset))
    {
	/*here the attacks take place*/
	int classID=strtol(row[2], NULL, 0);
	
	if(classID == 1)
	{
	    /*-------------------------------------------------
	      luckily its just a delivery - unload and get home
	      -------------------------------------------------*/
	    if(!unload(row, currentDB))
		return -1;
	
	}
	
	if(classID==2)
	{
	    /*---------------------------------------------
	      agressor wants to destroy as much as possible
	      ---------------------------------------------*
	    int** defenders;
    	    */
	    
	    list *root;
	    
	    /*first get defenders of target*/
	    if(! get_defenders(row, root, currentDB))
		return -1;
	    
	    /*fight*/
	    if(! fight(root, row, currentDB))
		return -1;
	    
	    if(! fly_home(row, currentDB))
		return 1;	
	}
	
	if(classID==3)
	{
	    MYSQL_RES* spy_result;
	    MYSQL_ROW spy_row;
	    
	    /*---------------------
	     visitor wants to spy
	     ----------------------*/
	    /*to do: set out the spyprobe and make the ship ready for return*/
	    
	    query=(char*) malloc(len=(strlen(Q750)+_MYSQL_INT_BYTELEN));
	    if(query==NULL)
	    {
		perror("malloc()");
		return -1;
	    }
	    
	    /*get all spyprobes in bundle in case of spying the planet*/
	    sprintf(query, Q750, row[1]);
	    if( (spy_result=(MYSQL_RES*)select_result(currentDB, query))==NULL)
	    {
		free(query);
	    	return -1;
	    }
	    free(query);
	    
	    while(spy_row=(MYSQL_ROW)mysql_fetch_row(spy_result))
	    {
		/*first change old attack order - set target to myOwnerID*/
		query=malloc(len=(strlen(Q751)+_MYSQL_INT_BYTELEN));
		if(query==NULL)
		{
		    perror("malloc()");
		    return -1;
		}
		
		sprintf(query, Q751, row[8]);
		if(!real_update_result(currentDB, query, len))
		    return -1;
		
		/*second delete spy from bundle*/
		query=malloc(len=(strlen(Q752)+_MYSQL_INT_BYTELEN));
		if(query==NULL)
		{
		    perror("malloc()");
		    return -1;
		}
		
		sprintf(query, Q752, row[1]);
		if(!real_update_result(currentDB, query, len))
		    return -1;
		
		/*third install spy in target's orbit*/
		query=malloc(len=(strlen(Q753)+_MYSQL_INT_BYTELEN));
		if(query==NULL)
		{
		    perror("malloc()");
		    return -1;
		}
		
		sprintf(query, Q753, row[3]);
		if(!real_update_result(currentDB, query, len))
		    return -1;
		
		/*forth update bundlenames setting current_pos to enemyID*/
		/*TODO for performance improvement - do this only once per flight*/
		query=malloc(len=(strlen(Q754)+_MYSQL_INT_BYTELEN));
		if(query==NULL)
		{
		    perror("malloc()");
		    return -1;
		}
		
		sprintf(query, Q754, row[3]);
		if(!real_update_result(currentDB, query, len))
		    return -1;
	    }
	    
	    /*attacks would work the following way: 
	     *
	     *	- check for spyprobes in  chars_troops with ownership=my.ownership and current_ownership=target.ownership
	     *  - show all planets visible as targetpossibilities
	     *  - in case of chosen target show its buildings etc in case of present spy on target
	     */
	}
	
	if(classID==4)
	{
	    /*------------------------------------------
	      visitor wants to take away our ressources
	      ------------------------------------------*/
	      
	    /*possibilities for stealing:
	    
	      1) attack whole planet
	         - choose all the stocks automatically
		 - start function plundering
		 
	      2) attack only certain stocks
	         - start function plundering */
	      
	      char* query;
	      long len;
	      MYSQL_RES* resultset;
	      MYSQL_ROW plunder_row;
	      
	      if(row[3])
	      {
		/*first get stocks on enemy planet*/
		query=malloc((len=sizeof(Q760) + _MYSQL_INT_BYTELEN));
		if(query==NULL)
		{
		    perror("malloc()");
		    return -1;
		}
		
		sprintf(query, Q760, row[3]);
		if( (resultset=select_result(currentDB,query))==NULL)
		{
		    free(query);
		    return -1;
		}
		free(query);
		
		/*then start plundering itself*/
		while(plunder_row=(MYSQL_ROW)mysql_fetch_row(resultset))
	    	    if(!start_plunder(plunder_row[0]), currentDB, row)
			return -1;
	      }
	      else
	        if(!start_plunder(row[4], currentDB, row))
		    return -1;
	
	      if(!fly_home(row, currentDB))
	        return -1;
	      
	}
    }	
    
    /*delete tmp*/
    if(! (administrate_db(currentDB, Q705) ))
	return -1;
	
    return 1;
}

int start_plunder(char* stock_to_plunder, MYSQL* currentDB, MYSQL_ROW row)
{
    
    list *root;
    int checker;
    
    if(!get_defenders(row, root, currentDB))
	return -1;
    
    /*insert target_stock into row*/
    row[4]=stock_to_plunder;
    
    if(! (checker= fight(root, row, currentDB)) )
	return -1;

    if(checker==1)
	if(!load_ship(currentDB, row[8], row[1]));

    return 1;
}

int load_ship(MYSQL* currentDB, char* ownerID_enemy, char* bundleID_plunderer)
{
    char* query;
    long len;
    long stock_count, ship_strength, ship_load;
    
    MYSQL_RES* content, *count, *strength;
    MYSQL_ROW content_row, count_row, strength_row;
    
    
    ship_load=0;
    
    /*step one: get content of one stock*/
    
    /*step one.one: get amount of stocks*/
    query=malloc(sizeof(Q762) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    
    sprintf(query, Q762, ownerID_enemy);
    if( (count=(MYSQL_RES*)select_result(currentDB, query))==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    if(count_row=(MYSQL_ROW)mysql_fetch_row(count))
    {
    	if(! (stock_count=strtol(count_row[0], NULL, 0)) )
	    return -1;
    }
    else
	stock_count=1;
	
    /*step one.two: get amount of all resources on colony*/
    query=malloc(sizeof(Q761) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }    
    
    sprintf(query, Q761, ownerID_enemy);
    if( (content=(MYSQL_RES*)select_result(currentDB, query)) ==NULL )
    {
	free(query);
    	return -1;
    }
    free(query);
    
    /*step one.three: get amount, ship can carry away*/
    query=malloc(sizeof(Q763)+_MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q763, bundleID_plunderer);
    
    if( (strength=(MYSQL_RES*)select_result(currentDB, query)) ==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    if( strength_row=(MYSQL_ROW)mysql_fetch_row(strength))
    {
    	if(! (ship_strength=strtol(strength_row[0], NULL, 0)) )
	    return -1;
    }
    else
	ship_strength=0;
    
    /*step one.four: get concrete content of each stock*/
    while(content_row=(MYSQL_ROW)mysql_fetch_row(content))
    {
	
	char* query;
	MYSQL_RES* weight_set, *ownerID_plunderer_set;
	MYSQL_ROW weight_row, ownerID_plunderer_row;
	
	long current_res_count, stock_res_count, plundered_count, res_weight;
	int checker;
	/*
	    0 ... resID
	    1 ... count
	*/
	
	/*get weight of resource to steal*/
	query=(char*) malloc(sizeof(Q766 + _MYSQL_INT_BYTELEN));
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q766, content_row[0]);
	if( (weight_set=select_result(currentDB, query))==NULL)
	{
	    free(query);
	    return -1;
	}
	free(query);
	
	if(! (weight_row=mysql_fetch_row(weight_set)) )
	    return -1;
	    
	if(! (res_weight=strtol(weight_row[0], NULL, 0)) )
	    return -1;
	
	/*get count of res to steal in stock targeted*/
	if(! (current_res_count=strtol(content_row[1], NULL, 0)) )
	    return -1;
	    
	/*compute content of one stock*/
	stock_res_count=current_res_count/stock_count;
	
	/*get amount ship can carry away*/
	plundered_count=stock_res_count;
	
	if(plundered_count*res_weight+ship_load > ship_strength)
	    plundered_count=(ship_strength-ship_load)/res_weight;
	
	/*save amount ship has already carried away*/
	ship_load+=plundered_count*res_weight;
	
	/*compute new content of colony-wide stock by substracting content of plundered stock*/
	current_res_count-=plundered_count;
	
	/*step two: update chars_troops_bundles by adding/increasing plundered resource*/
	/*two.one: get ownerID of bundle plundering*/
	query=(char*) malloc(sizeof(Q767)+_MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	
	sprintf(query, Q767, bundleID_plunderer);
	if( (ownerID_plunderer_set=select_result(currentDB, query)) ==NULL)
	{
	    free(query);
	    return -1;
	}
	free(query);
	
	if(! (ownerID_plunderer_row=mysql_fetch_row(ownerID_plunderer_set)) )
	    return -1;	    
	
	/*two.two: try using update*/
	query=(char*) malloc( len=(sizeof(Q764)+2*_MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q764, plundered_count, bundleID_plunderer, content_row[0]);
	
	/*two.three: try insert in case of no success*/
	if( (checker=real_update_result(currentDB, query,len)) == 0 )
	{
	    /*on plundering ship there were not yet any resources - insert new line*/
	    free(query);
	    query=(char*)malloc( len=(sizeof(Q765) + 4*_MYSQL_INT_BYTELEN) );
	    if(query==NULL)
	    {
		perror("malloc()");
		return -1;
	    }
	    
	    sprintf(query, Q765, ownerID_plunderer_row[0], content_row[0], bundleID_plunderer, plundered_count);
	    if(! (checker=real_update_result(currentDB, query, len)))
	    {
		/*could even not insert a new line into the db*/
		free(query);
		return -1;
	    }
	    
	}
	else
	{
	    free(query);
	    
	    if(checker==-1)
		return -1;
	}
	
	/*step three: take away resources from plundered*/
	query=(char*) malloc(len=(sizeof(Q768) + 3*_MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	
	sprintf(query, Q768, plundered_count, ownerID_enemy, content_row[0]);
	if(!real_update_result(currentDB, query, len))
	{
	    free(query);
	    return -1;
	}
	else 
	    free(query);
    }
    
    return 1;

    
}

long compute_impact(long max_impact)
{
    long min_impact;

    min_impact=(long)rint(max_impact/8);
    srand(time(NULL));
    
    max_impact=min_impact+ (int) (max_impact*rand()/(RAND_MAX+1.0));
    return max_impact;    
}

int p_for_dodge(char* target_troopID, MYSQL* currentDB)
{
    char* query;
    MYSQL_ROW row;
    MYSQL_RES* res;
    long agility;
    
    query=malloc(sizeof(Q7449) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q7449, target_troopID);
    if(! (res=select_result(currentDB, query)) )
    {
	free(query);
	return -1;
    }
    free(query);
    
    if(!(row=mysql_fetch_row(res)) )
	return -1;
    
    agility=(float)strtod(row[0], NULL);
    
    srand(time(NULL));
    return((int) rint( (rand()/RAND_MAX+1.0)*agility ) );
}

int attack_building(char* target, char* attID, list* walker, MYSQL* currentDB)
{
    char* query;
    long len, state, real_impact;
    MYSQL_ROW building_row;
    MYSQL_RES* building_res;
    
    /*get state of attacked building*/
    query=(char*)malloc(sizeof(Q7441) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q7441, target);
    
    if( (building_res=select_result(currentDB, query)) ==NULL)
    {
	free(query);
    	return -1;
    }
    free(query);
    building_row=mysql_fetch_row(building_res);
    
    state=strtol(building_row[0], NULL, 0);
    
    if(state-(real_impact=compute_impact(walker->strength_impact)) < 0)
    {
	int i;
	
	/*building has been destroyed - delete it*/
	query=(char*)malloc(sizeof(Q7442) + _MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7442, target);
	if(!update_result(currentDB, query))
	{
	    free(query);
	    return -1;
	}
	free(query);
	
	/*delete the building's influence onto the planet*/
	for(i=1; i<5; i++)
	{
	    query=(char*)malloc(strlen(Q7441b) + 3* _MYSQL_INT_BYTELEN);
	    if(query==NULL)
	    {
		perror("malloc");
		return -1;
	    }
	    sprintf(query, Q7441b, building_row[i], i-5, building_row[5]);
	    if(! (update_result(currentDB, query)))
	    {
		free(query);
		return -1;
	    }
	    free(query);
	}    
    }
    else
    {
	/*building has been hit - update state*/
	query=(char*)malloc(sizeof(Q7443) + 2*_MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7443,real_impact, target);
	if(!update_result(currentDB, query))
	{
	    free(query);
	    return -1;
	}
	free(query);
    }
    return 1;
}

int attack_build_inProcess(char* target, char* attID, list* walker, MYSQL* currentDB)
{
    char* query;
    long len, state, real_impact;
    MYSQL_ROW building_row;
    MYSQL_RES* building_res;
    
    /*get state of attacked building*/
    query=(char*)malloc(sizeof(Q7442a) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q7442a, target);
    
    if( (building_res=select_result(currentDB, query)) ==NULL)
    {
	free(query);
    	return -1;
    }
    free(query);
    building_row=mysql_fetch_row(building_res);
    
    /*ooooooooooooooooooooooooooooooooooooooooooooooooooooooo
	freshly build buildings get a bonus of one lifepoint
      ooooooooooooooooooooooooooooooooooooooooooooooooooooooo*/    
      
    state=strtol(building_row[0], NULL, 0)+1;
    
    if(state-(real_impact=compute_impact(walker->strength_impact)) < 0)
    {
	/*building has been destroyed - delete it*/
	query=(char*)malloc(sizeof(Q7442b) + _MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7442b, target);
	if(!update_result(currentDB, query))
	{
	    free(query);
	    return -1;
	}
	free(query);
    }
    else
    {
	/*building has been hit - update state*/
	query=(char*)malloc(sizeof(Q7442c) + 2*_MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7442c,  real_impact, target);
	if(! (update_result(currentDB, query)))
	{
	    free(query);
	    return -1;
	}
	free(query);
    }
    return 1;
    
}

int attack_troop(char* target, char* attID, list* walker, MYSQL* currentDB)
{
    char* query;
    long len, state, real_impact;
    MYSQL_ROW building_row;
    MYSQL_RES* building_res;
    
    /*get state of attacked troop*/
    query=(char*)malloc(sizeof(Q7444) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q7444, target);
    
    if( (building_res=select_result(currentDB, query)) ==NULL)
    {
	free(query);
    	return -1;
    }
    free(query);
    building_row=mysql_fetch_row(building_res);
    
    state=strtol(building_row[0], NULL, 0);

    if(p_for_dodge(target, currentDB))
	return 1;

    if(state-(real_impact= compute_impact(walker->strength_impact)) < 0)
    {
	/*building has been destroyed - delete it*/
	query=(char*)malloc(sizeof(Q7445) + _MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7445, target);
	if(!update_result(currentDB, query))
	{
	    free(query);
	    return -1;
	}
	free(query);
    }
    else
    {
	/*building has been hit - update state*/
	query=(char*)malloc(sizeof(Q7446) + 2*_MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7446,  real_impact, target);
	if( !update_result(currentDB, query))
	{
	    free(query);
	    return -1;
	}
	free(query);
    }
    return 1;
}

int attack_bundle(char* target, char* attID, list* walker, MYSQL* currentDB)
{
    char* query;
    MYSQL_ROW row;
    MYSQL_RES* res;
    
    query=malloc(sizeof(Q7448) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    
    sprintf(query, Q7448, target);
    if((res=select_result(currentDB, query))==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    if(!(row=mysql_fetch_row(res)) )
	return -1;
    
    if(p_for_dodge(row[0], currentDB))
	return 1;

    if(!distribute_impact( compute_impact(walker->strength_impact), target, currentDB))
	return -1;
	
    return 1;
}

int distribute_impact(long impact, char* victim, MYSQL* currentDB)
{
    MYSQL_ROW row;
    MYSQL_RES* resultset;
    long len, shielded, plated, ship_impact, carrier_state;
    char* query, *carrierID;
    list* hit, *walker;
    int hit_objects=0;
    
    /*get all weapons on the ship (all except the ship itself)*/
    query=(char*)malloc(sizeof(Q743) +_MYSQL_INT_BYTELEN);
    sprintf(query, Q743, victim);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    
    if( (resultset=select_result(currentDB, query)) ==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    shielded=plated=0;
    while(row=mysql_fetch_row(resultset))
    {
	if(strtol(row[12], NULL, 0) !=3)
	{
    	    list* newFighter=create_new_fighter(row, 1);
	    in_order(hit, newFighter);
	
	    /*compute the defence of the ship
	     *
	     *defense is stored in state of the fighter
	     */
	    if(strtol(row[12], NULL,0)==1)
		shielded+=newFighter->state;
	    if(strtol(row[12], NULL,0)==2)
		plated+=newFighter->state;
	
	    /*get sum of attacked objects*/
	    hit_objects++;
	}
	else
	{
	    carrierID=row[0];
	    carrier_state=strtol(row[4], NULL, 0);
	}
    }
    
    impact-=shielded;
    
    impact-=plated;
    
    /*
     ooooooooooooooooooooooooooooooooooooooooooooooooooooooo
    
         Formel zur Berechnung des
	 impacts auf jede einzelne
	 Komponente!
	 
	 Um eine hoehere Bewertung
	 des Einschlags auf das 
	 Schiff zu erreichen:
	 
	 impact=(impact-Schiffsbewertung)/hit_objects
    
     ooooooooooooooooooooooooooooooooooooooooooooooooooooooo
     */
    ship_impact=impact=impact/(hit_objects+1);
    
    walker=hit;
    while(walker!=NULL)
    {
	if(walker->state-impact <= 0)
	{
	    /*troop attached has been destroyed
	     * delete:
	     *	entry in chars_troops
	     *	entry in chars_troops_bundles
	     */
	     
	     /*delete troop*/
	     query=(char*) malloc( len=(sizeof(Q7445) +_MYSQL_INT_BYTELEN));
	     if(query==NULL)
	     {
	        perror("malloc()");
		return -1;
	     }
	     
	     sprintf(query,Q7445, victim);
	     if(!real_update_result(currentDB, query, len))
	     {
	        free(query);
		return -1;
	     }
	     free(query);
	     
	     /*delete bundle entry*/
	     query=(char*)malloc( len=(sizeof(Q7445a +_MYSQL_INT_BYTELEN)) );
	     if(query==NULL)
	     {
	        perror("malloc()");
		return -1;
	     }
	     sprintf(query, Q7445a, victim);
	     if(!real_update_result(currentDB, query, len))
	     {
	        free(query);
		return -1;
	     }
	     free(query);
	     
	     walker=walker->next;
	     
	     /*delete died walker*/
	     free(walker);
	}
	else
	{
	    /*in case part of victim is still alive after impact: decrease its state*/
	    walker->state-=impact;
	}
    }
    
    if(!handle_impact_on_carrier(hit, victim, ship_impact, carrierID, carrier_state, currentDB))
	return -1;
	
    return 1;
}

int handle_impact_on_carrier(list* hit, char* victim, int ship_impact, char* carrierID, int carrier_state, MYSQL* currentDB)
{
    list* walker;
    char* query;
    long len;
    
    
    carrier_state-=ship_impact;
    if(carrier_state <= 0)
    {
	/*uff, we have to delete the whole bundle
	 *
	 *    affects:	chars_troops		 	7445d OK
	 *              chars_troops.carrier	 	7445d
	 *		chars_troops_bundles 	 	7445b OK
	 *		chars_troops_bundlenames 	7445c OK
	 *		chars_troops_attacks     	7445e OK
	 */
	 
	while(hit!=NULL)
	{
	    /*delete fighter from chars_troops*/
	    query=malloc( len=(sizeof(Q7445d) + _MYSQL_INT_BYTELEN) );
	    if(query==NULL)
	    {
		perror("malloc()");
		return -1;
	    }
	    sprintf(query, Q7445d, hit->troopID);
	    if(!real_update_result(currentDB,query, len))
	    {
		free(query);
		return -1;
	    }	    
	    free(query);
	    
	    
	    walker=hit;
	    hit=hit->next;
	    free(walker);
	}
	 
	/*delete bundleparts*/
	query=malloc( len=(sizeof(Q7445b) + _MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7445b, victim);
	if(!real_update_result(currentDB,query, len))
	{
	    free(query);
	    return -1;
	}	    
	free(query);

	/*delete from bundlenames*/
	query=malloc( len=(sizeof(Q7445c) + _MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7445c, victim);
	if(!real_update_result(currentDB,query, len))
	{
	    free(query);
	    return -1;
	}	    
	free(query);

	/*delete attacks*/
	query=malloc( len=(sizeof(Q7445e) + _MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7445e, victim);
	if(!real_update_result(currentDB,query, len))
	{
	    free(query);
	    return -1;
	}	    
	free(query);

	/*delete carrier*/
	query=malloc( len=(sizeof(Q7445d) + _MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7445d, carrierID);
	if(!real_update_result(currentDB,query, len))
	{
	    free(query);
	    return -1;
	}	    
	free(query);
	return 2;
    }
    else
    {
	/*carrier has not been killed - reduce its state*/
	query=malloc( len=(sizeof(Q7446) + _MYSQL_INT_BYTELEN) );
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q7446, ship_impact, carrierID);
	if(!real_update_result(currentDB,query, len))
	{
	    free(query);
	    return -1;
	}	    
	free(query);
    }    
    return 1;
}

int attack_widely(char* target, list* walker, list* root, MYSQL* currentDB)
{
    char *query;
    long len, x, y;
    MYSQL_ROW row;
    MYSQL_RES* resultset;
    
    /*get place where impact might happen - at the moment
      target searching radar does not have any effect*/
      
    query=(char*) malloc(sizeof(Q74473) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
     
    sprintf(query, Q74473, target);
    if( (resultset=select_result(currentDB,query))==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    if(!(row=mysql_fetch_row(resultset)) )
	return -1;
	
    /*x and y for the range the impact could happen in*/
    x=strtol(row[0], NULL,0)+strtol(row[2], NULL, 0);
    y=strtol(row[1], NULL,0)+strtol(row[2], NULL, 0);
    
    /*actual x and y*/
    srand(time(NULL));
    x=1+ (int) (x*rand()/(RAND_MAX+1.0));
    y=1+ (int) (y*rand()/(RAND_MAX+1.0));
    
    /*get troop on this place*/
    query=(char*) malloc(sizeof(Q74473a) + 3* _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
     
    sprintf(query, Q74473a, x, y, target);
    if( (resultset=select_result(currentDB, query))==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    if(row=mysql_fetch_row(resultset))
    {
	MYSQL_ROW bundle_row;
	
	/*check whether troop is carrier of a bundle*/
        query=(char*) malloc(sizeof(Q74473d) + 3* _MYSQL_INT_BYTELEN);
        if(query==NULL)
        {
    	    perror("malloc()");
	    return -1;
        }
         
        sprintf(query, Q74473a, x, y, target);
        if( (resultset=select_result(currentDB, query))==NULL)
        {
    	    free(query);
    	    return -1;
        }
        free(query);
        
        if(bundle_row=mysql_fetch_row(resultset))
    	{
	    /*bundle has been hit*/
	    if(!attack_bundle(bundle_row[0], NULL, walker, currentDB))
		return -1;
	}
	else
	    if(!attack_troop(row[0], NULL, walker, currentDB))
		return -1;
    }
    else
    {
	/*get building on this place*/
        query=(char*) malloc(sizeof(Q74473b) + 3* _MYSQL_INT_BYTELEN);
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
     
	sprintf(query, Q74473b,x, y, target);
	if( (resultset=select_result(currentDB, query))==NULL)
	{
	    free(query);
	    return -1;
	}
	free(query);
    
	if(row=mysql_fetch_row(resultset))
	{
	    attack_building(row[0], NULL, walker, currentDB);
	}
	else
	{

	    /*get building in process on this place*/
    	    query=(char*) malloc(sizeof(Q74473c) + 3* _MYSQL_INT_BYTELEN);
	    if(query==NULL)
	    {
		perror("malloc()");
		return -1;
	    }
     
	    sprintf(query, Q74473c, x,y,target);
	    if( (resultset=select_result(currentDB,query))==NULL)
	    {
		free(query);
		return -1;
	    }
	    free(query);
    
	    if(row=mysql_fetch_row(resultset))
	    {
		if(!attack_build_inProcess(row[0], NULL, walker, currentDB))
		    return -1;
	    }
	    /* else no hit was made*/
	    
	}
    }
    
    /*kick defenders which can no more reach ship*/
    
    walker=root;
    while(walker!=NULL)
	if( sqrt( pow((walker->x-x),2)+pow((walker->y-y),2) +pow((walker->z-0),2) ) < walker->reach )
	    /*cick defender*/
	    delete_fighter(root, walker, currentDB);	    
    return 1;
}

int fight (list *root, MYSQL_ROW row, MYSQL* currentDB)
{
    list *defender, *agressor, *walker, *helper;
    long shielded, plated, state_carrier;
    char* carrierID;
    int strike_count;
    
    if(! (get_aggressors(root, row, currentDB, &shielded, &plated)) )
	return -1;
	
    walker=root;
    
    strike_count=-1;    
    
    /*first strike*/
    while(walker!=NULL)
    {
	strike_count++;
	
	if(!walker->defender)
        {
    	    /*strike of agressor on building to attack*/
	    /*
	        0 attID
	        1 myBundleID
	        2 classID
	        3 ownerID_target
    	        4 buildID_enemy
	        5 troopID_enemy
		6 bundleID_enemy
	        7 sequence_number for checks or whatever
	        8 ownerID_current_pos to make life easiear
	    */
	    
	    /*do I want to attack a normal building*/
	    if(row[4])
	    {
	        if(!attack_building(row[4], row[0], walker, currentDB))
	    	    return -1;
	    }
	    /*do I want to attack a troop*/
	    if(row[5])
	    {
	        if(!attack_troop(row[5], row[0], walker, currentDB))
		    return -1;
	    }
	    
	    /*do I want to attack a bundle*/
	    if(row[6])
	    {
		if(!attack_bundle(row[6], row[0], walker, currentDB))
		    return -1;
	    }
	
	    /*do I want to attack the whole planet*/
	    if(row[3])
	    {
	        if(!attack_widely(row[3], walker, root, currentDB))
		    return -1;
	    }
	
	    /*do I want to attack something on a certain coordinate still has to be implemented in database
	    if(row[]){}*/
	}
        else
	{
	    /*strike of defender onto agressor*/
	    
	    /*
	    ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
	    changed state as reference for the force of the shields etc
	    ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
	     */
	    char* query_c;
	    MYSQL_ROW row_c;
	    MYSQL_RES* res_c;
	 
	    int ship_impact;
	    int real_strike_impact= compute_impact(walker->strength_impact);
	    real_strike_impact-=shielded;

    
	    query_c=malloc(sizeof(Q7448) + _MYSQL_INT_BYTELEN);
	    if(query_c==NULL)
	    {
		perror("malloc()");
		return -1;
	    }
    
	    sprintf(query_c, Q7448, row[1]);
	    if((res_c=select_result(currentDB, query_c))==NULL)
	    {
		free(query_c);
		return -1;
	    }
	    free(query_c);
	    if(!(row_c=mysql_fetch_row(res_c)) )
		return -1;
		
	    if(real_strike_impact > 0 && !p_for_dodge(row_c[0], currentDB))
	    {
		int remaining_plate_protection=plated-real_strike_impact;
	        real_strike_impact-=plated;
	    
	        plated=remaining_plate_protection;
	    
	        if(real_strike_impact > 0)
	        {
	    	    /*defender actually has succeeded and now can hurt aggressor*/
		    /*Vorgehen aehnlich wie in distribute impact, nur ohne db-zugriffe*/
		    list* hunter;
		    list* rabbit, *rabbits_father;
		    int agg_number=0;
		    MYSQL_ROW delete_row;
		    MYSQL_RES* delete_set;
		    char* query;
		    long len;
		
		    hunter=walker;
		    rabbit=root;
		
		    /*find number of aggressors*/
		    while(rabbit!=NULL)
		    {
			if(!rabbit->defender)
			{
			    /*we have found an aggressor*/
			    agg_number++;
			}
			rabbit=rabbit->next;
		    }
		    real_strike_impact=real_strike_impact/(agg_number+1);
		    ship_impact=real_strike_impact;
		
		    /*distribute strike on each aggressor*/
		    rabbit=root;
		    while(rabbit!=NULL)
		    {
		        if(!rabbit->defender)
			{
			    if(rabbit->state-real_strike_impact < 0)
		    	    {
				/*rabbit is destroyed*/
			
				/*delete rabbit from chars_troops*/
				query=(char*)malloc(len=(_MYSQL_INT_BYTELEN+sizeof(Q7445d)) );
				if(query==NULL)
				{
				    perror("malloc()");
			    	    return -1;
				}
				sprintf(query, Q7445d, rabbit->troopID);
			        if(!real_update_result(currentDB, query, len))
				{
				    free(query);
			    	    return -1;
				}
			    
			        /*delete rabbit from chars_troops_bundles*/
				query=(char*)malloc(len=(_MYSQL_INT_BYTELEN+sizeof(Q7445a)) );
			        if(query==NULL)
		    		{
			            perror("malloc()");
			            return -1;
			        }
			        sprintf(query, Q7445a, rabbit->troopID);
			        if(!real_update_result(currentDB, query, len))
			        {
			            free(query);
			            return -1;
			        }
			
		    		/*delete rabbit from list of aggressors*/
				if(!rabbits_father)
				    root=rabbit->next;
				else
				    rabbits_father->next=rabbit->next;
				free(rabbit);
			
			    }
		    	    else
		    	    {
		    	        /*rabbit is hit*/
		    		rabbit->state-=real_strike_impact;
			    
				/*write hit into db for dataconsistency - can be omitted and done at the end of the
				  fight in case of performance-problems*/
			      
				query=(char*)malloc(len=(_MYSQL_INT_BYTELEN+sizeof(Q7446)) );
				if(query==NULL)
		    		{
			    	    perror("malloc()");
			    	    return -1;
				}
				sprintf(query, Q7446, real_strike_impact, rabbit->troopID);
				if(!real_update_result(currentDB, query, len))
				{
			            free(query);
				    return -1;
				}
			    
			    }
			}
	    		rabbits_father=rabbit;
			rabbit=rabbit->next;
		    }
		    {
		        char* query_txt;
		        MYSQL_RES* res;
		        MYSQL_ROW line;
		        list *hit, *die, *die_next, *die_prev, *new_hit, *walker;
		        int checker;
		    
		        /*everything is hit except the ship itself*/
		        query_txt=(char*) malloc(sizeof(Q7448) + _MYSQL_INT_BYTELEN);
		        if(query_txt==NULL)
		        {
		            perror("malloc()");
		            return -1;
		        }
		        sprintf(query_txt, Q7448, row[1]);
		        if( ((res=select_result(currentDB, query_txt))==NULL) )
		        {
		        	free(query_txt);
				return -1;
		        }
		        free(query_txt);
		        
		        if(! (line=mysql_fetch_row(res)))
		        	return -1;
		    	
		        carrierID=line[0];
		        state_carrier=strtol(line[1], NULL, 0);
		        
		        walker=root;
		        while(walker!=NULL)
		        {
				if(!walker->defender)
		        	{
				    new_hit=copy_fighter(walker);
		    	    
				    if(!in_order(hit, new_hit))
					return -1;
				}
		        }
		        
		        checker=handle_impact_on_carrier(hit, carrierID, ship_impact, carrierID, state_carrier, currentDB);
		        if(checker==-1)
		        	return -1;
			
		        if(checker==2)
		        {
    	        	    /*delete all aggressors from fightlist*/
		    	    /*they have been deleted from db already*/
			    die=die_next=root;
			    die_prev=NULL;
			    
			    while(die!=NULL)
			    {
				die_next=die->next;
			        if(!die->defender)
				{
			    	    if(die_prev!=NULL)
					die_prev->next=die->next;
				    free(die);
				}
				die=die_next;
			    }
			}	
		    }/*end of block for destruction of carrier*/
		}/*end of real_strike_impact after plateinfluence*/
	    }/*end of real_strike_impact*/
	}/*end of else (if!(walker->defender))*/
	
	/*in case current weapon can not shoot anymore - kick it out of the list*/
	helper=walker->next;
	if(walker->strike_count<strike_count)
	    delete_fighter(root, walker, currentDB);
	walker=helper;	
    }/*end while*/

    return 1;
}

int fly_home(MYSQL_ROW row, MYSQL* currentDB)
{
    char* query;
    long len;
    
    /*content of row:
    	0 attID
	1 myBundleID
	2 classID
	3 ownerID_target
	4 buildID_enemy
	5 troopID_enemy
	6 bundleID_enemy
	7 sequence_number for checks or whatever
	8 ownerID_current_pos to make life easiear
    */    
    
    /*first generate flight home*/
    query=(char*) malloc(len=(sizeof(Q771)+2*_MYSQL_INT_BYTELEN));
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q771, row[1], row[8]);
    if(!real_update_result(currentDB, query, len))
    {
	free(query);
	return -1;
    }
    free(query);
    
    /*delete old attack*/
    query=(char*) malloc(len=(sizeof(Q772)+ _MYSQL_INT_BYTELEN));
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q772, row[0]);
    if(!real_update_result(currentDB, query, len))
    {
	free(query);
	return -1;
    }
    free(query);
    
    return 1;
}

int get_defenders(MYSQL_ROW row, list *root, MYSQL* currentDB)
{
    MYSQL_RES* resultset;
    MYSQL_ROW current_row;
    char* query, *x, *y, *z;
    long len;
    int position, i;
    
    /*
	get postion of bundle attacking
	
	in case it attacks the whole planet, all enemies are accepted to defeat their home
	
	in case attacker gets first strike, all enemies not near the chosen target are ommitted from the list
	in case defender gets first strike, all power can be used to defeat aggressor, defenders are omitted at point of
	 time the agressor is striking

	3 ownerID_target
	4 buildID_enemy
	5 troopID_enemy
	6 bundleID_enemy

    */
    
    if(!row[3])
    {
	if(row[4])
	{
    	    query=(char*) malloc( (strlen(Q740b1) + _MYSQL_INT_BYTELEN));
    	    if(query==NULL)
	    {
		perror("malloc()");
	        return -1;
	    }
	
	    sprintf(query, Q740b1, row[4]);
	    if((resultset=select_result(currentDB, query))==NULL)
	    {
		free(query);
		return -1;
	    }
	    free(query);
    
	    if(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
	    {
		x=current_row[0];
		y=current_row[1];
		z=current_row[2];
	    }
	    else return -1;
	}
	
	if(row[5])
	{
    	    query=(char*) malloc( (strlen(Q740b2) + _MYSQL_INT_BYTELEN));
    	    if(query==NULL)
	    {
		perror("malloc()");
	        return -1;
	    }
	
	    sprintf(query, Q740b2, row[5]);
	    if((resultset=select_result(currentDB, query))==NULL)
	    {
		free(query);
		return -1;
	    }
	    free(query);
    
	    if(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
	    {
		x=current_row[0];
		y=current_row[1];
		z=current_row[2];
	    }
	    else return -1;

	}
	
	if(row[6])
	{
    	    query=(char*) malloc( (strlen(Q740b3) + _MYSQL_INT_BYTELEN));
    	    if(query==NULL)
	    {
		perror("malloc()");
	        return -1;
	    }
	
	    sprintf(query, Q740b3, row[8]);
	    if((resultset=select_result(currentDB, query))==NULL)
	    {
		free(query);
		return -1;
	    }
	    free(query);
    
	    if(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
	    {
		x=current_row[0];
		y=current_row[1];
		z=current_row[2];
	    }
	    else return -1;
	}
	query=(char*) malloc( (strlen(Q740b) + _MYSQL_INT_BYTELEN));
        if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
    
	sprintf(query, Q740b, row[8], current_row[0], current_row[1], current_row[2]);
    }
    else
    {
	query=(char*) malloc( (strlen(Q740a) + _MYSQL_INT_BYTELEN));
        if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
    
	sprintf(query, Q740a, row[8]);

    }    
    /*get all free troops on enemyownership*/
    if((resultset=select_result(currentDB, query))==NULL)
    {
	free(query);
	return -1;
    }
    free(query);
    
    while(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
    {
	/*first create a new node holding the defender*/
	
	list* newFighter=create_new_fighter(current_row, 1);
	
	in_order(root, newFighter);
    
    }
    return 1;
}

int get_aggressors(list* root, MYSQL_ROW row, MYSQL* currentDB, long *shielded, long* plated)
{
    MYSQL_RES* resultset;
    MYSQL_ROW current_row;
    char* query;
    long len;
    
    query=(char*)malloc( sizeof(Q743) + _MYSQL_INT_BYTELEN);
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    sprintf(query, Q743, row[1]);
    if( (resultset=select_result(currentDB, query))==NULL)
	return -1;

    free(query);
    
    while(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
    {
	if(strtol(row[12], NULL, 0) != 3)
	{
	    /*we do not want to have the carrier as fighter!*/
	    list* newFighter=create_new_fighter(current_row, 0);
	
	    in_order(root, newFighter);
	
	    if(strtol(current_row[12], NULL,0)==1)
		*shielded+=newFighter->state;
	    if(strtol(current_row[12], NULL,0)==2)
	        *plated+=newFighter->state;
	}
    }
    return 1;
}

int unload(MYSQL_ROW row, MYSQL* currentDB)
{
    MYSQL_RES* resultset;
    MYSQL_ROW current_row;
    char* query;
    long len;
    
    /*resources are in the bundle - get them, transfer them to the property of the planetowner*/
    if( (resultset=select_result(currentDB, Q710)) == NULL)
	return -1;
    /*
	0 bundle_partID
	1 resID
	2 count
    */
    while(current_row=(MYSQL_ROW) mysql_fetch_row(resultset))
    {
        /*increase stock of receiver*/
	query=(char*) malloc( (len=strlen(Q711) + 3*_MYSQL_INT_BYTELEN));
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	
	sprintf(query, Q711, current_row[2], current_row[1], row[3]);
	if(! real_update_result(currentDB, query, len))
	    return -1;
	free(query);
	
	/*delete resources from ship*/
	query=(char*) malloc( (len=strlen(Q712) + _MYSQL_INT_BYTELEN));
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	
	sprintf(query, Q712, current_row[0]);
	if(! real_update_result(currentDB, query, len))
	    return -1;
	free(query);
    }

    /*in case there are other troops in the bundle - get them and make them be the property of the planetowner*/
    if( (resultset=select_result(currentDB, Q713)) == NULL)
	return -1;
    /*
	0 bundle_partID
	1 troopID
    */
    
    while(current_row=(MYSQL_ROW) mysql_fetch_row(resultset))
    {
	query=(char*) malloc( (len=strlen(Q714) + _MYSQL_INT_BYTELEN));
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q714, current_row[1]);
	if(!real_update_result(currentDB, query, len))
	    return -1;
	free(query);
	
	/*delete troop from bundle*/
	query=(char*) malloc( (len=strlen(Q715) + _MYSQL_INT_BYTELEN));
	if(query==NULL)
	{
	    perror("malloc()");
	    return -1;
	}
	sprintf(query, Q715, current_row[0]);
	if(! real_update_result(currentDB, query, len))
	    return -1;
	free(query);
    }
    
    /*update bundle set current_ownerPos*/
    query=(char*) malloc( (len=strlen(Q73) + _MYSQL_INT_BYTELEN));
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    	
    sprintf(query, Q73, row[3]);
    if(! real_update_result(currentDB, query, len))
	return -1;
    free(query);
    
    /*delete flight from chars_troops_attacks*/
    query=(char*) malloc( (len=strlen(Q72) +_MYSQL_INT_BYTELEN));
    if(query==NULL)
    {
        perror("malloc()");
        return -1;
    }
    
    sprintf(query, Q72, row[0]);
    if(! real_update_result(currentDB, query, len))
        return -1;
    free(query);
    
}

/*deprecated get_defenders !!!*

int get_defenders(MYSQL_ROW row, int** defenders, MYSQL* currentDB)
{
    MYSQL_RES* resultset;
    MYSQL_ROW current_row;
    char* query;
    long len;
    int position, i;
    
    /*get all free troops on enemyownership*
    query=(char*) malloc( (strlen(Q740) + _MYSQL_INT_BYTELEN));
    if(query==NULL)
    {
	perror("malloc()");
	return -1;
    }
    
    sprintf(query, Q740, row[8]);
    if((resultset=select_result(currentDB, query))==NULL)
	return -1;
    free(query);
    
    position=0;
    while(current_row=(MYSQL_ROW)mysql_fetch_row(resultset))
    {
        int **defenders=(int**) realloc (defenders, (position+1)*sizeof(int*));
	if(defenders==NULL)
        {
    	    perror("malloc()");
	    return -1;
	}

	defenders[position]=(int*) malloc(_DEFENDERS_PROPERTIES_LENGTH_*sizeof(int));
	
	for(i=0; i<_DEFENDERS_PROPERTIES_LENGTH_; i++)
	    defenders[position][i]=strtol(current_row[i], NULL, 0);
	position++;
    }
    return 1;
}
*/

    /*
      one fight considers one attacker - each ship attacking is treated separatly
      
      Defenders do not contain ships already loaded - bundles - have to be treated separatly
      
      At the moment we fight until the state of the ship is below 2.
      
      Another reason to stop is that there are no more attacks to do - each attack takes place 1 time at the moment (could become more when changing the
      interface automatically).
      
      For fleeing defenders there will be an extra funktion deleting them from the array and updating their state in the database.
    */
    
    
    /*Preparation of fight
     *
     * get parameters of carrier and bundle into one array containing:
     *  > state of carrier     					(is being decreased during the fight in order to find out when ship will give up)
     *  > shielded = sum( troop_type(class.Schild)).strength_impact ) (decreases the impact of each hit - remains during fight as being the result of a shield)
     *  > defended = sum( troop_type(class.Panzerung)).strength_impact) (decreases the impact of each hit - is decreased in the same intense the impact takes place)
     
     * put all attached weapons into a double array where each line contains:
     *  > state of weapon                - decreased at each hit
     *  > strength_impact		 - to make strikes possible
     *  > agility			 - to define the initiative of the weapon and the probablity of a hit (the larger the more initiative and the smaller the hitprobablity)
     *  > reach				 - to define whether it is possible to attack one certain opponent
     
     * get one empty array where to store information about defenders having had their strike
     *  > pointer to line in defender array who has striked
     *  > impact that actually hurt aggressor
     
     * for the choice whom to attack next 
     *  > pointer to defense-row in array
     *  > number representing the last hitpoints fired on the bundle
     *
    **/
    
    /*step one - check reach of defender 
     *
     *in case of target_is_reachable:
     *
     *	REPEAT
     *	step two - find out who strikes first
     *  step two.one - strike of defender
     *		> take next defender which can reach the opponent
     *			REPEAT AGILITY_MAX-agility.fach
     *			> take strength_impact
     *			> substract strength_impact from shielded of me 		strenth_impact-=shielded)
     *   		> substract remaining strength_impact from defended of me	remaining_impact=strength_impact-defended;
     *											defended-=strength_impact-remaining_impact;
     *			> distribute remaining_impact on attached weapons and ship	for_all(remaining_weapons)
     *											if(prob(0...1)*agility>0.5)	
     *											  	state.weapon-=remaining_impact/count(weapons)
     *			> generate entry in defenders_striked 
     *			END REPEAT
     *
     *	step two.two - strike of aggressor
     *		> take next attached weapon - in case of reached end of weapons take the first again
     *			REPEAT AGILITY_MAX-agility.fach 
     * 			> choose reachable opponent from list of defenders having shot already otherwise take target
     *			> take strength_impact of weapon
     *			> substract impact from state (protection here is impossible)
     *			> delete entry from defenders_striked in case it was a defence against them
     *                  END REPEAT
     *  
     *	UNTIL (target destroyed OR carrier's state below MIN_STATE)
     
    AGILITY should reach from 0 to 10 until now - needs define for agility_MAX
    
    both defenders and attached_weapons could maybe better stored in list in the form of a ring
    
    */
